package cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.base;

import cn.estsh.i3plus.ext.mes.api.base.IMesConfigService;
import cn.estsh.i3plus.ext.mes.pojo.util.MesExtConstWords;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.mes.bean.MesConfig;
import cn.estsh.i3plus.pojo.mes.repository.MesConfigRepository;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @Description:
 * @CreateDate: 2021/9/9 下午3:25
 * @Author: simon.song
 */
@Service
@Slf4j
public class MesConfigService implements IMesConfigService {
    @Autowired
    private MesConfigRepository configRepository;

    @Override
    public String getCfgValue(String org, String key) {
        MesConfig config = configRepository.getByProperty(
                new String[]{"cfgKey", "organizeCode", "isValid", "isDeleted"},
                new Object[]{key, org, CommonEnumUtil.IS_VAILD.VAILD.getValue(), CommonEnumUtil.IS_DEAL.NO.getValue()});

        if (config == null) {
            throw ImppExceptionBuilder.newInstance().setErrorCode(cn.estsh.i3plus.platform.common.exception.ImppExceptionEnum.BUSINESS_EXCEPTION_DATA_ERROR.getCode())
                    .setErrorDetail(String.format("配置表数据不存在，请维护key【%s】", key))
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                    .build();
        }
        return config.getCfgValue();
    }

    @Override
    public MesConfig getCfgValueByCode(String cfgCode) {
        return this.configRepository.getByProperty(new String[]{"isDeleted", "isValid", "cfgCode"},
                new Object[]{CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue(), CommonEnumUtil.IS_VAILD.VAILD.getValue(), cfgCode});
    }

    @Override
    public Boolean checkCfgValueIsContainItem(String organizeCode, String cfgCode, String cfgKey, String cfgType, String splitStr, String item) {
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(organizeCode);
        DdlPreparedPack.getStringEqualPack(cfgCode, MesExtConstWords.CFG_CODE, packBean);
        DdlPreparedPack.getStringEqualPack(cfgKey, MesExtConstWords.CFG_KEY, packBean);
        DdlPreparedPack.getStringEqualPack(cfgType, MesExtConstWords.CFG_TYPE, packBean);
        MesConfig config = configRepository.getByProperty(packBean);
        if (null == config || StringUtils.isEmpty(config.getCfgValue())) return false;
        List<String> cfgValueList = Arrays.asList(config.getCfgValue().split(splitStr));
        return (!CollectionUtils.isEmpty(cfgValueList) && cfgValueList.contains(item)) ? true : false;
    }

    @Override
    public Boolean checkShardingTablesObjectCfg(String organizeCode, String cfgKey, String item) {
        return checkCfgValueIsContainItem(organizeCode, MesExtConstWords.MES_SHARDING_TABLES_OBJECT_CFG, cfgKey, CommonEnumUtil.SOFT_TYPE.MES.name(), MesExtConstWords.SEMICOLON, item);
    }

    @Override
    public List<String> findCfgValueContainItem(String organizeCode, String cfgCode, String cfgKey, String cfgType, String splitStr, List<String> itemList) {
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(organizeCode);
        DdlPreparedPack.getStringEqualPack(cfgCode, MesExtConstWords.CFG_CODE, packBean);
        DdlPreparedPack.getStringEqualPack(cfgKey, MesExtConstWords.CFG_KEY, packBean);
        DdlPreparedPack.getStringEqualPack(cfgType, MesExtConstWords.CFG_TYPE, packBean);
        MesConfig config = configRepository.getByProperty(packBean);
        if (null == config || StringUtils.isEmpty(config.getCfgValue())) return null;
        List<String> cfgValueList = Arrays.asList(config.getCfgValue().split(splitStr));
        if (CollectionUtils.isEmpty(cfgValueList)) return null;
        return itemList.stream().filter(cfgValueList::contains).collect(Collectors.toList());
    }

    @Override
    public List<String> findShardingTablesObjectCfg(String organizeCode, String cfgKey, List<String> itemList) {
        if(CollectionUtils.isEmpty(itemList)) return null;
        return findCfgValueContainItem(organizeCode, MesExtConstWords.MES_SHARDING_TABLES_OBJECT_CFG, cfgKey, CommonEnumUtil.SOFT_TYPE.MES.name(), MesExtConstWords.SEMICOLON, itemList);
    }

}
